@@ -165,7 +165,7 @@ class Node:
             pass
         if not res:
             logger.fatal('failed to receive a response from the ark-node')
-            raise NodeDbError('failed to receive a response from the ark-node')
+            raise NodeDbError('failed to receive a response from the ark-node: {}'.format(e))
         return res
 
     @staticmethod
@@ -205,7 +205,7 @@ class Address:
                        transactions."timestamp", transactions."recipientId",
                        transactions."senderId", transactions."rawasset",
                        transactions."type", transactions."fee"
-                FROM transactions
+                FROM transactions, delegates
                 WHERE transactions."senderId" IN (
                   SELECT transactions."senderId" 
                   FROM transactions, delegates 
@@ -272,12 +272,12 @@ class Address:
 
     @staticmethod
     def votes(address):
-        """Returns a map all votes made by an address, {(+/-)pubkeydelegate:timestamp}, timestamp DESC"""
+        """Returns a list of namedtuples all votes made by an address, {(+/-)pubkeydelegate:timestamp}, timestamp DESC"""
         qry = DbCursor().execute_and_fetchall("""
            SELECT votes."votes", transactions."timestamp"
            FROM votes, transactions
            WHERE transactions."id" = votes."transactionId"
-           AND transactions."recipientId" = '{}'
+           AND transactions."senderId" = '{}'
            ORDER BY transactions."timestamp" DESC
         """.format(address))
 
@@ -303,7 +303,8 @@ class Address:
 
     @staticmethod
     def balance(address):
-        """Takes a single address and returns the current balance.
+        """
+        Takes a single address and returns the current balance.
         """
         txhistory = Address.transactions(address)
         balance = 0
@@ -319,6 +320,7 @@ class Address:
                 forged_blocks = Delegate.blocks(i.pubkey)
                 for block in forged_blocks:
                     balance += (block.reward + block.totalFee)
+
         if balance < 0:
             height = Node.height()
             logger.fatal('Negative balance for address {0}, Nodeheight: {1)'.format(address, height))
@@ -342,19 +344,13 @@ class Address:
         Balance = namedtuple(
             'balance',
             'timestamp amount')
+
         for tx in txhistory:
             if forged_blocks:
                 while forged_blocks[block].timestamp <= tx.timestamp:
-                    block += 1
                     balance += (forged_blocks[block].reward + forged_blocks[block].totalFee)
-                    res = Balance(timestamp=forged_blocks[block].timestamp, amount=balance)
-                    balance_over_time.append(res)
-
-                if forged_blocks[block].timestamp > txhistory[len(txhistory) - 1].timestamp:
-                    for i in forged_blocks[block:]:
-                        balance += (i.reward + i.totalFee)
-                        res = Balance(timestamp=i.timestamp, amount=balance)
-                        balance_over_time.append(res)
+                    balance_over_time.append(Balance(timestamp=forged_blocks[block].timestamp, amount=balance))
+                    block += 1
 
             if tx.senderId == address:
                 balance -= (tx.amount + tx.fee)
@@ -364,6 +360,14 @@ class Address:
                 balance += tx.amount
                 res = Balance(timestamp=tx.timestamp, amount=balance)
                 balance_over_time.append(res)
+
+        if forged_blocks and block <= len(forged_blocks) - 1:
+            if forged_blocks[block].timestamp > txhistory[-1].timestamp:
+                for i in forged_blocks[block:]:
+                    balance += (i.reward + i.totalFee)
+                    res = Balance(timestamp=i.timestamp, amount=balance)
+                    balance_over_time.append(res)
+
         return balance_over_time
 
 
@@ -628,89 +632,95 @@ class Delegate:
         block_nr = start_block
         chunk_dict = {}
         reuse = False
-        for tx in transactions:
-            while tx.timestamp > blocks[block_nr].timestamp:
-                if reuse:
+        try:
+            for tx in transactions:
+                while tx.timestamp > blocks[block_nr].timestamp:
+                    if reuse:
+                        block_nr += 1
+                        for x in chunk_dict:
+                            voter_dict[x]['share'] += chunk_dict[x]
+                        continue
                     block_nr += 1
-                    for x in chunk_dict:
-                        voter_dict[x]['share'] += chunk_dict[x]
-                    continue
-                block_nr += 1
-                poolbalance = 0
-                chunk_dict = {}
-                for i in voter_dict:
-                    balance = voter_dict[i]['balance']
+                    poolbalance = 0
+                    chunk_dict = {}
+                    for i in voter_dict:
+                        balance = voter_dict[i]['balance']
+
+                        try:
+                            if voter_dict[i]['balance'] > c.CALCULATION_SETTINGS['MAX']:
+                                balance = c.CALCULATION_SETTINGS['MAX']
+                        except Exception:
+                            pass
+
+                        try:
+                            if balance > c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']:
+                                balance = c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']
+                        except Exception:
+                            pass
+
+                        try:
+                            for x in voter_dict[i]['blocks_forged']:
+                                if x.timestamp < blocks[block_nr].timestamp:
+                                    voter_dict[i]['balance'] += (x.reward + x.totalFee)
+                                    voter_dict[i]['blocks_forged'].remove(x)
+                            balance = voter_dict[i]['balance']
+                        except Exception:
+                            pass
+
+                        if voter_dict[i]['status']:
+                            if not voter_dict[i]['balance'] < -20 * c.ARK:
+                                poolbalance += balance
+                            else:
+                                logger.fatal('balance lower than zero for: {0}'.format(i))
+                                raise NegativeBalanceError('balance lower than zero for: {0}'.format(i))
+
+                    for i in voter_dict:
+                        balance = voter_dict[i]['balance']
 
-                    try:
                         if voter_dict[i]['balance'] > c.CALCULATION_SETTINGS['MAX']:
                             balance = c.CALCULATION_SETTINGS['MAX']
-                    except Exception:
-                        pass
 
-                    try:
-                        if balance > c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']:
-                            balance = c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']
-                    except Exception:
-                        pass
+                        try:
+                            if balance > c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']:
+                                balance = c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']
+                        except Exception:
+                            pass
 
-                    try:
-                        for x in voter_dict[i]['blocks_forged']:
-                            if x.timestamp < blocks[block_nr].timestamp:
-                                voter_dict[i]['balance'] += (x.reward + x.totalFee)
-                                voter_dict[i]['blocks_forged'].remove(x)
-                        balance = voter_dict[i]['balance']
-                    except Exception:
-                        pass
-
-                    if voter_dict[i]['status']:
-                        if not voter_dict[i]['balance'] < 0:
-                            poolbalance += balance
-                        else:
-                            logger.fatal('balance lower than zero for: {0}'.format(i))
-                            raise NegativeBalanceError('balance lower than zero for: {0}'.format(i))
-
-                for i in voter_dict:
-                    balance = voter_dict[i]['balance']
-
-                    if voter_dict[i]['balance'] > c.CALCULATION_SETTINGS['MAX']:
-                        balance = c.CALCULATION_SETTINGS['MAX']
-
-                    try:
-                        if balance > c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']:
-                            balance = c.CALCULATION_SETTINGS['EXCEPTIONS'][i]['REPLACE']
-                    except Exception:
-                        pass
-
-                    if voter_dict[i]['status'] and voter_dict[i]['last_payout'] < blocks[block_nr].timestamp:
-                        if c.CALCULATION_SETTINGS['SHARE_FEES']:
-                            share = (balance/poolbalance) * (blocks[block_nr].reward +
-                                                             blocks[block_nr].totalFee)
-                        else:
-                            share = (balance/poolbalance) * blocks[block_nr].reward
-                        voter_dict[i]['share'] += share
-                        chunk_dict.update({i: share})
-                reuse = True
+                        if voter_dict[i]['status'] and voter_dict[i]['last_payout'] < blocks[block_nr].timestamp:
+                            if c.CALCULATION_SETTINGS['SHARE_FEES']:
+                                share = (balance/poolbalance) * (blocks[block_nr].reward +
+                                                                 blocks[block_nr].totalFee)
+                            else:
+                                share = (balance/poolbalance) * blocks[block_nr].reward
+                            voter_dict[i]['share'] += share
+                            chunk_dict.update({i: share})
+                    reuse = True
 
 
-            # parsing a transaction
-            minvote = '{{"votes":["-{0}"]}}'.format(delegate_pubkey)
-            plusvote = '{{"votes":["+{0}"]}}'.format(delegate_pubkey)
+                # parsing a transaction
+                minvote = '{{"votes":["-{0}"]}}'.format(delegate_pubkey)
+                plusvote = '{{"votes":["+{0}"]}}'.format(delegate_pubkey)
 
-            reuse = False
+                reuse = False
 
-            if tx.recipientId in voter_dict:
-                voter_dict[tx.recipientId]['balance'] += tx.amount
-            if tx.senderId in voter_dict:
-                voter_dict[tx.senderId]['balance'] -= (tx.amount + tx.fee)
-            if tx.senderId in voter_dict and tx.type == 3 and plusvote in tx.rawasset:
-                voter_dict[tx.senderId]['status'] = True
-            if tx.senderId in voter_dict and tx.type == 3 and minvote in tx.rawasset:
-                voter_dict[tx.senderId]['status'] = False
+                if tx.recipientId in voter_dict:
+                    voter_dict[tx.recipientId]['balance'] += tx.amount
+                if tx.senderId in voter_dict:
+                    voter_dict[tx.senderId]['balance'] -= (tx.amount + tx.fee)
+                if tx.senderId in voter_dict and tx.type == 3 and plusvote in tx.rawasset:
+                    voter_dict[tx.senderId]['status'] = True
+                if tx.senderId in voter_dict and tx.type == 3 and minvote in tx.rawasset:
+                    voter_dict[tx.senderId]['status'] = False
+
+            remaining_blocks = len(blocks) - block_nr - 1
+            for i in range(remaining_blocks):
+                for x in chunk_dict:
+                    voter_dict[x]['share'] += chunk_dict[x]
+
+        # an IndexError occurs if max(transactions.timestamp) > max(blocks.timestamp) This means we parsed every block
+        except IndexError:
+            pass
 
-        remaining_blocks = len(blocks) - block_nr - 1
-        for i in range(remaining_blocks):
-            for x in chunk_dict:
-                voter_dict[x]['share'] += chunk_dict[x]
         for i in voter_dict:
             logger.info("{0}  {1}  {2}  {3}  {4}".format(i,
                                                          voter_dict[i]['share'],
@@ -728,9 +738,11 @@ class Delegate:
 
         the only option it evaluates is SHARE_FEES
 
+        One caveat: the maximum interval of 6.8 minutes between a transaction and a forged block by the delegate can
+        provide errors. There always needs to be a forged block
+
         '''
         logger.info('starting share calculation using settings: {0} {1}'.format(c.DELEGATE, c.CALCULATION_SETTINGS))
-        cursor = DbCursor()
 
         if passphrase:
             delegate_keys = core.getKeys(secret=passphrase,
@@ -771,6 +783,13 @@ class Delegate:
                 'blocks_forged': []}
             })
 
+        try:
+            for i in c.CALCULATION_SETTINGS['BLACKLIST']:
+                voter_dict.pop(i)
+                logger.debug('popped {} from calculations'.format(i))
+        except Exception:
+            pass
+
         # check if a voter is/used to be a forging delegate
         delegates = Delegate.delegates()
         for i in delegates:
@@ -795,76 +814,80 @@ class Delegate:
         block_nr = start_block
         chunk_dict = {}
         reuse = False
-        for tx in transactions:
-            while tx.timestamp > blocks[block_nr].timestamp:
-                if reuse:
+        try:
+            for tx in transactions:
+                while tx.timestamp > blocks[block_nr].timestamp:
+                    if reuse:
+                        block_nr += 1
+                        for x in chunk_dict:
+                            voter_dict[x]['share'] += chunk_dict[x]
+                        continue
+
                     block_nr += 1
-                    for x in chunk_dict:
-                        voter_dict[x]['share'] += chunk_dict[x]
-                    continue
-                block_nr += 1
-                poolbalance = 0
-                chunk_dict = {}
-                for i in voter_dict:
-                    balance = voter_dict[i]['balance']
-
-                    #checks if a delegate that votes for us is has forged blocks in the mean time
-                    try:
-                        for x in voter_dict[i]['blocks_forged']:
-                            if x.timestamp < blocks[block_nr].timestamp:
-                                voter_dict[i]['balance'] += (x.reward + x.totalFee)
-                                voter_dict[i]['blocks_forged'].remove(x)
+                    poolbalance = 0
+                    chunk_dict = {}
+                    for i in voter_dict:
                         balance = voter_dict[i]['balance']
-                    except Exception:
-                        pass
 
-                    if voter_dict[i]['status']:
-                        if not voter_dict[i]['balance'] < 0:
-                            poolbalance += balance
-                        else:
-                            logger.fatal('balance lower than zero for: {0}'.format(i))
-                            raise NegativeBalanceError('balance lower than zero for: {0}'.format(i))
+                        #checks if a delegate that votes for us is has forged blocks in the mean time
+                        try:
+                            for x in voter_dict[i]['blocks_forged']:
+                                if x.timestamp < blocks[block_nr].timestamp:
+                                    voter_dict[i]['balance'] += (x.reward + x.totalFee)
+                                    voter_dict[i]['blocks_forged'].remove(x)
+                            balance = voter_dict[i]['balance']
+                        except Exception:
+                            pass
 
-                for i in voter_dict:
-                    balance = voter_dict[i]['balance']
+                        if voter_dict[i]['status']:
+                            if not voter_dict[i]['balance'] < -20 * c.ARK:
+                                poolbalance += balance
+                            else:
+                                logger.fatal('balance lower than zero for: {0}'.format(i))
+                                raise NegativeBalanceError('balance lower than zero for: {0}'.format(i))
 
-                    if voter_dict[i]['status'] and voter_dict[i]['last_payout'] < blocks[block_nr].timestamp:
-                        if c.CALCULATION_SETTINGS['SHARE_FEES']:
-                            share = (balance / poolbalance) * (blocks[block_nr].reward +
-                                                               blocks[block_nr].totalFee)
-                        else:
-                            share = (balance / poolbalance) * blocks[block_nr].reward
-                        voter_dict[i]['share'] += share
-                        chunk_dict.update({i: share})
-                reuse = True
+                    for i in voter_dict:
+                        balance = voter_dict[i]['balance']
 
-            # parsing a transaction
-            minvote = '{{"votes":["-{0}"]}}'.format(delegate_pubkey)
-            plusvote = '{{"votes":["+{0}"]}}'.format(delegate_pubkey)
+                        if voter_dict[i]['status'] and voter_dict[i]['last_payout'] < blocks[block_nr].timestamp:
+                            if c.CALCULATION_SETTINGS['SHARE_FEES']:
+                                share = (balance / poolbalance) * (blocks[block_nr].reward +
+                                                                   blocks[block_nr].totalFee)
+                            else:
+                                share = (balance / poolbalance) * blocks[block_nr].reward
+                            voter_dict[i]['share'] += share
+                            chunk_dict.update({i: share})
+                    reuse = True
 
-            reuse = False
+                # parsing a transaction
+                minvote = '{{"votes":["-{0}"]}}'.format(delegate_pubkey)
+                plusvote = '{{"votes":["+{0}"]}}'.format(delegate_pubkey)
 
-            if tx.recipientId in voter_dict:
-                voter_dict[tx.recipientId]['balance'] += tx.amount
-            if tx.senderId in voter_dict:
-                voter_dict[tx.senderId]['balance'] -= (tx.amount + tx.fee)
-            if tx.senderId in voter_dict and tx.type == 3 and plusvote in tx.rawasset:
-                voter_dict[tx.senderId]['status'] = True
-            if tx.senderId in voter_dict and tx.type == 3 and minvote in tx.rawasset:
-                voter_dict[tx.senderId]['status'] = False
+                reuse = False
 
+                if tx.recipientId in voter_dict:
+                    voter_dict[tx.recipientId]['balance'] += tx.amount
+                if tx.senderId in voter_dict:
+                    voter_dict[tx.senderId]['balance'] -= (tx.amount + tx.fee)
+                if tx.senderId in voter_dict and tx.type == 3 and plusvote in tx.rawasset:
+                    voter_dict[tx.senderId]['status'] = True
+                if tx.senderId in voter_dict and tx.type == 3 and minvote in tx.rawasset:
+                    voter_dict[tx.senderId]['status'] = False
 
-        remaining_blocks = len(blocks) - block_nr - 1
-        for i in range(remaining_blocks):
-            for x in chunk_dict:
-                voter_dict[x]['share'] += chunk_dict[x]
+            remaining_blocks = len(blocks) - block_nr - 1
+            for i in range(remaining_blocks):
+                for x in chunk_dict:
+                    voter_dict[x]['share'] += chunk_dict[x]
+        except IndexError:
+            pass
 
         for i in voter_dict:
-            logger.info("{0}  {1}  {2}  {3}  {4}".format(i,
-                                                         voter_dict[i]['share'],
-                                                         voter_dict[i]['status'],
-                                                         voter_dict[i]['last_payout'],
-                                                         voter_dict[i]['vote_timestamp']))
+            logger.info("{0}  {1}  {2}  {3}  {4}".format(
+                i,
+                voter_dict[i]['share'],
+                voter_dict[i]['status'],
+                voter_dict[i]['last_payout'],
+                voter_dict[i]['vote_timestamp']))
         return voter_dict, max_timestamp
 
 
@@ -1020,7 +1043,4 @@ def get_transactionlist(delegate_pubkey):
                             )
 
         named_transactions.append(tx_id)
-    return named_transactions
-
-def new():
-    print('helloworld')
\ No newline at end of file
+    return named_transactions
\ No newline at end of file
